@@ -1,11 +1,13 @@
 use clap::clap_app;
 use crate::error::RadError;
 use crate::processor::Processor;
+use crate::logger::DebugOption;
 use std::path::{Path, PathBuf};
 
 /// Struct to parse command line arguments and execute proper operations
 pub struct Cli{
     rules: Option<Vec<PathBuf>>,
+    debug_options: Option<Vec<DebugOption>>,
     write_to_file : Option<PathBuf>,
     error_to_file : Option<PathBuf>,
 }
@@ -15,6 +17,7 @@ impl Cli {
     pub fn new() -> Self {
         Self {
             rules: None,
+            debug_options : None,
             write_to_file : None,
             error_to_file : None,
         }
@@ -41,6 +44,7 @@ impl Cli {
             .custom_rules(std::mem::replace(&mut self.rules,None))?
             .write_to_file(std::mem::replace(&mut self.write_to_file,None))?
             .error_to_file(std::mem::replace(&mut self.error_to_file,None))?
+            .debug(std::mem::replace(&mut self.debug_options,None))?
             .build();
 
         // ========
@@ -86,15 +90,20 @@ impl Cli {
         } else { None };
 
         // Error to file 
-        self.error_to_file = if let Some(output_file) = args.value_of("error") {
-            Some(PathBuf::from(output_file))
+        self.error_to_file = if let Some(error_file) = args.value_of("error") {
+            Some(PathBuf::from(error_file))
+        } else { None };
+
+        self.debug_options = if let Some(debug_options) = args.value_of("debug") {
+            let option_vec = debug_options.split('+').map(|s| DebugOption::new(s)).collect::<Vec<DebugOption>>();
+            Some(option_vec)
         } else { None };
     }
 
     /// Creates argument template
     fn args_builder() -> clap::ArgMatches {
         clap_app!(rad =>
-            (version: "0.6.0")
+            (version: "0.7.0")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
@@ -105,6 +114,7 @@ impl Cli {
             (@arg freeze: -f +takes_value "Freeze to file")
             (@arg purge: -p "Purge unused macros")
             (@arg strict: -S "Strict mode")
+            (@arg debug: -d --debug +takes_value "Debug mode [break|log|lines]")
             (@arg combination: -c "Read from both stdin and file inputs")
             (@arg silent: -s "Supress error and warning")
             (@arg newline: -n "Use unix newline for formatting")
