@@ -7,9 +7,11 @@ pub(crate) mod lexor;
 pub(crate) mod models;
 pub(crate) mod utils;
 pub(crate) mod arg_parser;
+pub(crate) mod logger;
 
 pub use error::RadError;
 pub use processor::Processor;
+pub use logger::DebugOption;
 
 // Optional
 
